@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
  */
-#ifndef HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
@@ -27,6 +27,13 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef ENABLE_NLS
+#  include <libintl.h>
+#else
+#  define textdomain(String) (String)
+#  define dgettext(Domain, String) (String)
+#endif
+
 #include <libxml/parser.h>
 
 #include "glade-parser.h"
@@ -81,6 +88,8 @@ typedef struct _GladeParseState GladeParseState;
 struct _GladeParseState {
     ParserState state;
 
+    const gchar *domain;
+
     guint unknown_depth;    /* handle recursive unrecognised tags */
     ParserState prev_state; /* the last `known' state we were in */
 
@@ -92,6 +101,7 @@ struct _GladeParseState {
 
     enum {PROP_NONE, PROP_WIDGET, PROP_ATK, PROP_CHILD } prop_type;
     gchar *prop_name;
+    gboolean translate_prop;
     GArray *props;
 
     GArray *signals;
@@ -355,6 +365,7 @@ glade_parser_start_document(GladeParseState *state)
 
     state->prop_type = PROP_NONE;
     state->prop_name = NULL;
+    state->translate_prop = FALSE;
     state->props = NULL;
 
     state->signals = NULL;
@@ -454,10 +465,14 @@ glade_parser_start_element(GladeParseState *state,
 		state->prop_type != PROP_WIDGET)
 		g_warning("non widget properties defined here (oh no!)");
 	    state->prop_type = PROP_WIDGET;
+	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
 		    state->prop_name = alloc_string(state->interface,
 						    attrs[i+1]);
+		else if (!strcmp(attrs[i], "translatable") &&
+			 !strcmp(attrs[i+1], "yes"))
+		    state->translate_prop = TRUE;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -497,10 +512,14 @@ glade_parser_start_element(GladeParseState *state,
 		state->prop_type != PROP_ATK)
 		g_warning("non atk properties defined here (oh no!)");
 	    state->prop_type = PROP_ATK;
+	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
 		    state->prop_name = alloc_string(state->interface,
 						    attrs[i+1]);
+		else if (!strcmp(attrs[i], "translatable") &&
+			 !strcmp(attrs[i+1], "yes"))
+		    state->translate_prop = TRUE;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -616,10 +635,14 @@ glade_parser_start_element(GladeParseState *state,
 		state->prop_type != PROP_CHILD)
 		g_warning("non child properties defined here (oh no!)");
 	    state->prop_type = PROP_CHILD;
+	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
 		    state->prop_name = alloc_string(state->interface,
 						    attrs[i+1]);
+		else if (!strcmp(attrs[i], "translatable") &&
+			 !strcmp(attrs[i+1], "yes"))
+		    state->translate_prop = TRUE;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -700,7 +723,12 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	prop.value = alloc_string(state->interface, state->content->str);
+	if (state->translate_prop) {
+	    prop.value = alloc_string(state->interface,
+			dgettext(state->domain, state->content->str));
+	} else {
+	    prop.value = alloc_string(state->interface, state->content->str);
+	}
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
 	state->state = PARSER_WIDGET;
@@ -717,7 +745,12 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	prop.value = alloc_string(state->interface, state->content->str);
+	if (state->translate_prop) {
+	    prop.value = alloc_string(state->interface,
+			dgettext(state->domain, state->content->str));
+	} else {
+	    prop.value = alloc_string(state->interface, state->content->str);
+	}
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
 	state->state = PARSER_WIDGET_ATK;
@@ -762,7 +795,12 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	prop.value = alloc_string(state->interface, state->content->str);
+	if (state->translate_prop) {
+	    prop.value = alloc_string(state->interface,
+			dgettext(state->domain, state->content->str));
+	} else {
+	    prop.value = alloc_string(state->interface, state->content->str);
+	}
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
 	state->state = PARSER_WIDGET_CHILD_PACKING;
@@ -792,8 +830,7 @@ glade_parser_characters(GladeParseState *state, const xmlChar *chars, int len)
     case PARSER_WIDGET_PROPERTY:
     case PARSER_WIDGET_ATK_PROPERTY:
     case PARSER_WIDGET_CHILD_PACKING_PROPERTY:
-	for (i = 0; i < len; i++)
-	    g_string_append_c(state->content, chars[i]);
+	g_string_append_len(state->content, chars, len);
 	break;
     default:
 	/* don't care about content in any other states */
@@ -884,6 +921,12 @@ widget_info_free(GladeWidgetInfo *info)
     g_free(info);
 }
 
+/**
+ * glade_interface_destroy
+ * @interface: the GladeInterface structure.
+ *
+ * Frees a GladeInterface structure.
+ */
 void
 glade_interface_destroy(GladeInterface *interface)
 {
@@ -907,12 +950,31 @@ glade_interface_destroy(GladeInterface *interface)
     g_free(interface);
 }
 
+/**
+ * glade_parser_parse_file
+ * @file: the filename of the glade XML file.
+ * @domain: the translation domain for the XML file.
+ *
+ * This function parses a Glade XML interface file to a GladeInterface
+ * object (which is libglade's internal representation of the
+ * interface data).
+ *
+ * Generally, user code won't need to call this function.  Instead, it
+ * should go through the GladeXML interfaces.
+ *
+ * Returns: the GladeInterface structure for the XML file.
+ */
 GladeInterface *
-glade_parser_parse_file(const gchar *file)
+glade_parser_parse_file(const gchar *file, const gchar *domain)
 {
     GladeParseState state = { 0 };
 
     state.interface = NULL;
+    if (domain)
+	state.domain = domain;
+    else
+	state.domain = textdomain(NULL);
+
     if (xmlSAXUserParseFile(&glade_parser, &state, file) < 0) {
 	g_warning("document not well formed!");
 	glade_interface_destroy(state.interface);
@@ -924,13 +986,33 @@ glade_parser_parse_file(const gchar *file)
     return state.interface;
 }
 
+/**
+ * glade_parser_parse_buffer
+ * @buffer: a buffer in memory containing XML data.
+ * @len: the length of @buffer.
+ * @domain: the translation domain for the XML file.
+ *
+ * This function is similar to glade_parser_parse_file, except that it
+ * parses XML data from a buffer in memory.  This could be used to
+ * embed an interface into the executable, for instance.
+ *
+ * Generally, user code won't need to call this function.  Instead, it
+ * should go through the GladeXML interfaces.
+ *
+ * Returns: the GladeInterface structure for the XML buffer.
+ */
 GladeInterface *
-glade_parser_parse_buffer(const gchar *buffer, gint size)
+glade_parser_parse_buffer(const gchar *buffer, gint len, const gchar *domain)
 {
     GladeParseState state = { 0 };
 
     state.interface = NULL;
-    if (xmlSAXUserParseMemory(&glade_parser, &state, buffer, size) < 0) {
+    if (domain)
+	state.domain = domain;
+    else
+	state.domain = textdomain(NULL);
+
+    if (xmlSAXUserParseMemory(&glade_parser, &state, buffer, len) < 0) {
 	g_warning("document not well formed!");
 	glade_interface_destroy(state.interface);
 	return NULL;
@@ -1060,6 +1142,14 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 	xmlNodeAddContent(widget, "  ");
 }
 
+/**
+ * glade_interface_dump
+ * @interface: the GladeInterface
+ * @filename: the filename to write the interface data to.
+ *
+ * This function dumps the contents of a GladeInterface into a file as
+ * XML.  It is intended mainly as a debugging tool.
+ */
 void
 glade_interface_dump(GladeInterface *interface, const gchar *filename)
 {
