@@ -41,7 +41,6 @@
 
 static const char *glade_xml_tag = "GladeXML::";
 static const char *glade_xml_name_tag = "GladeXML::name";
-static const char *glade_xml_longname_tag = "GladeXML::longname";
 
 static void glade_xml_init(GladeXML *xml);
 static void glade_xml_class_init(GladeXMLClass *class);
@@ -52,8 +51,6 @@ static void glade_xml_finalize(GObject *object);
 static void glade_xml_build_interface(GladeXML *xml, GladeInterface *iface,
 				      const char *root);
 
-GladeExtendedFunc *glade_xml_build_extended_widget = NULL;
-
 /**
  * glade_xml_get_type:
  *
@@ -102,13 +99,11 @@ glade_xml_init (GladeXML *self)
     self->priv = priv = g_new (GladeXMLPrivate, 1);
 
     self->filename = NULL;
-    self->txtdomain = NULL;
 
     priv->tree = NULL;
     priv->tooltips = gtk_tooltips_new();
     gtk_tooltips_enable(priv->tooltips);
     priv->name_hash = g_hash_table_new(g_str_hash, g_str_equal);
-    priv->longname_hash = g_hash_table_new(g_str_hash, g_str_equal);
     priv->signals = g_hash_table_new(g_str_hash, g_str_equal);
     priv->radio_groups = g_hash_table_new (g_str_hash, g_str_equal);
     priv->toplevel = NULL;
@@ -122,6 +117,7 @@ glade_xml_init (GladeXML *self)
  * glade_xml_new:
  * @fname: the XML file name.
  * @root: the widget node in @fname to start building from (or %NULL)
+ * @domain: the translation domain for the XML file (or %NULL for default)
  *
  * Creates a new GladeXML object (and the corresponding widgets) from the
  * XML file @fname.  Optionally it will only build the interface from the
@@ -137,8 +133,8 @@ glade_xml_new(const char *fname, const char *root, const char *domain)
 {
     GladeXML *self = g_object_new(GLADE_TYPE_XML, NULL);
 
-    if (!glade_xml_construct(self, fname, root, NULL)) {
-	gtk_object_unref(GTK_OBJECT(self));
+    if (!glade_xml_construct(self, fname, root, domain)) {
+	g_object_unref(G_OBJECT(self));
 	return NULL;
     }
     return self;
@@ -168,7 +164,7 @@ glade_xml_new_with_domain(const char *fname, const char *root,
     GladeXML *self = g_object_new(GLADE_TYPE_XML, NULL);
 
     if (!glade_xml_construct(self, fname, root, domain)) {
-	gtk_object_unref(GTK_OBJECT(self));
+	g_object_unref(G_OBJECT(self));
 	return NULL;
     }
     return self;
@@ -195,14 +191,12 @@ glade_xml_construct (GladeXML *self, const char *fname, const char *root,
     g_return_val_if_fail(self != NULL, FALSE);
     g_return_val_if_fail(fname != NULL, FALSE);
 
-    iface = glade_parser_parse_file(fname);
+    iface = glade_parser_parse_file(fname, domain);
 
     if (!iface)
 	return FALSE;
 
     self->priv->tree = iface;
-    if (self->txtdomain) g_free(self->txtdomain);
-    self->txtdomain = g_strdup(domain);
     if (self->filename)
 	g_free(self->filename);
     self->filename = g_strdup(fname);
@@ -231,14 +225,13 @@ glade_xml_new_from_buffer(const char *buffer, int size, const char *root,
 			  const char *domain)
 {
     GladeXML *self;
-    GladeInterface *iface = glade_parser_parse_buffer(buffer, size);
+    GladeInterface *iface = glade_parser_parse_buffer(buffer, size, domain);
 
     if (!iface)
 	return NULL;
     self = g_object_new(GLADE_TYPE_XML, NULL);
 
     self->priv->tree = iface;
-    self->txtdomain = g_strdup(domain);
     self->filename = NULL;
     glade_xml_build_interface(self, iface, root);
 
@@ -581,29 +574,6 @@ glade_xml_get_widget_prefix (GladeXML *self, const gchar *prefix)
     return data.list;
 }
 
-/**
- * glade_xml_get_widget_by_long_name:
- * @self: the GladeXML object.
- * @longname: the long name of the widget (eg toplevel.parent.widgetname).
- *
- * This function is used to get a pointer to the GtkWidget corresponding to
- * @longname in the interface description.  You would use this if you have
- * to do anything to the widget after loading.  This function differs from
- * glade_xml_get_widget, in that you have to give the long form of the
- * widget name, with all its parent widget names, separated by periods.
- *
- * Returns: the widget matching @longname, or %NULL if none exists.
- */
-GtkWidget *
-glade_xml_get_widget_by_long_name(GladeXML *self,
-				  const char *longname)
-{
-    g_return_val_if_fail(self != NULL, NULL);
-    g_return_val_if_fail(longname != NULL, NULL);
-
-    return g_hash_table_lookup(self->priv->longname_hash, longname);
-}
-
 /**
  * glade_xml_relative_file
  * @self: the GladeXML object.
@@ -649,25 +619,6 @@ glade_get_widget_name(GtkWidget *widget)
 					     glade_xml_name_tag);
 }
 
-/**
- * glade_get_widget_long_name:
- * @widget: the widget
- *
- * Description:
- * Used to get the long name of a widget that was generated by a GladeXML
- * object.
- *
- * Returns: the long name of the widget.
- */
-const char *
-glade_get_widget_long_name (GtkWidget *widget)
-{
-    g_return_val_if_fail(widget != NULL, NULL);
-
-    return (const char *)gtk_object_get_data(GTK_OBJECT(widget),
-					     glade_xml_longname_tag);
-}
-
 /**
  * glade_get_widget_tree:
  * @widget: the widget
@@ -721,7 +672,7 @@ glade_xml_set_toplevel(GladeXML *xml, GtkWindow *window)
 			     (GtkDestroyNotify)gtk_object_unref);
 }
 
- /**
+/**
  * glade_xml_push_accel:
  * @xml: the GladeXML object.
  *
@@ -738,7 +689,8 @@ glade_xml_push_accel(GladeXML *xml)
     return accel;
 }
 
-/* glade_xml_pop_accel:
+/**
+ * glade_xml_pop_accel:
  * @xml: the GladeXML object.
  *
  * Pops the accel group.  This will usually be called after a GtkNotebook page
@@ -839,33 +791,6 @@ glade_xml_get_parent_accel(GladeXML *xml)
 }
 #endif
 
-/**
- * glade_xml_gettext:
- * @xml: the GladeXML object.
- * @msgid: the string to translate.
- *
- * This function is a wrapper for gettext, that uses the translation domain
- * requested by the user of the function, or the default if no domain has
- * been specified.  This should be used for translatable strings in all
- * widget building routines.
- *
- * Returns: the translated string
- */
-char *
-glade_xml_gettext(GladeXML *xml, const char *msgid)
-{
-    if (!msgid || msgid[0] == '\0')
-	return "";
-#ifdef ENABLE_NLS
-    if (xml->txtdomain)
-	return dgettext(xml->txtdomain, msgid);
-    else
-	return gettext(msgid);
-#else
-    return msgid;
-#endif
-}
-
 /* this is a private function */
 static void
 glade_xml_add_signals(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
@@ -923,26 +848,31 @@ free_radio_groups(gpointer key, gpointer value, gpointer user_data)
     g_free(key);  /* the string name */
 }
 
+static void
+remove_data_func(gpointer key, gpointer value, gpointer user_data)
+{
+    GObject *object = value;
+    GladeXML *xml = user_data;
+
+    g_object_set_data(object, glade_xml_tag, NULL);
+    g_object_set_data(object, glade_xml_name_tag, NULL);
+}
+
 static void
 glade_xml_finalize(GObject *object)
 {
     GladeXML *self = GLADE_XML(object);
     GladeXMLPrivate *priv = self->priv;
 	
-    if (self->filename)
-	g_free(self->filename);
+    g_free(self->filename);
     self->filename = NULL;
-    if (self->txtdomain)
-	g_free(self->txtdomain);
-    self->txtdomain = NULL;
 
     if (priv) {
-	if (priv->tree)
-	    glade_interface_destroy(priv->tree);
-	/* strings are owned in the cached GladeWidgetTree structure */
+	/* remove data items from all the widgets that are still
+         * live. */
+	g_hash_table_foreach(priv->name_hash, remove_data_func, self);
+	/* strings are owned in the GladeInterface structure */
 	g_hash_table_destroy(priv->name_hash);
-	/* strings belong to individual widgets -- don't free them */
-	g_hash_table_destroy(priv->longname_hash);
 
 	g_hash_table_foreach(priv->signals,
 			     (GHFunc)glade_xml_destroy_signals, NULL);
@@ -960,6 +890,9 @@ glade_xml_finalize(GObject *object)
 	if (priv->accel_groups)
 	    glade_xml_pop_accel(self);
 
+	if (priv->tree)
+	    glade_interface_destroy(priv->tree);
+
 	g_free (self->priv);
     }
     self->priv = NULL;
@@ -1033,7 +966,7 @@ glade_create_custom(GladeXML *xml, gchar *func_name, gchar *name,
 
 /**
  * glade_enum_from_string
- * @type: the GtkType for this flag or enum type.
+ * @type: the GType for this enum type.
  * @string: the string representation of the enum value.
  *
  * This helper routine is designed to be used by widget build routines to
@@ -1066,6 +999,19 @@ glade_enum_from_string (GType type, const char *string)
     return ret;
 }
 
+/**
+ * glade_flags_from_string
+ * @type: the GType for this flags type.
+ * @string: the string representation of the flags value.
+ *
+ * This helper routine is designed to be used by widget build routines
+ * to convert the string representations of flags values found in the
+ * XML descriptions to the integer values that can be used to
+ * configure the widget.  The string is composed of string names or
+ * nicknames for various flags separated by '|'.
+ *
+ * Returns: the integer value for this flags string
+ */
 guint
 glade_flags_from_string (GType type, const char *string)
 {
@@ -1117,7 +1063,7 @@ glade_xml_build_interface(GladeXML *self, GladeInterface *iface,
     if (root) {
 	wid = g_hash_table_lookup(iface->names, root);
 	g_return_if_fail(wid != NULL);
-	w = glade_xml_build_widget(self, wid, NULL);
+	w = glade_xml_build_widget(self, wid);
 	if (GTK_IS_WINDOW(w)) {
 	    if (self->priv->focus_widget)
 		gtk_widget_grab_focus(self->priv->focus_widget);
@@ -1127,7 +1073,7 @@ glade_xml_build_interface(GladeXML *self, GladeInterface *iface,
     } else {
 	/* build all toplevel nodes */
 	for (i = 0; i < iface->n_toplevels; i++) {
-	    w = glade_xml_build_widget(self, iface->toplevels[i],NULL);
+	    w = glade_xml_build_widget(self, iface->toplevels[i]);
 	}
 	if (self->priv->focus_widget)
 	    gtk_widget_grab_focus(self->priv->focus_widget);
@@ -1168,6 +1114,23 @@ glade_register_widgets(const GladeWidgetBuildData *widgets)
     }
 }
 
+/**
+ * glade_xml_set_value_from_string
+ * @pspec: the GParamSpec for the property
+ * @string: the string representation of the value.
+ * @value: the GValue to store the result in.
+ *
+ * This function demarshals a value from a string.  This function
+ * calls g_value_init() on the @value argument, so it need not be
+ * initialised beforehand.
+ *
+ * This function can handle char, uchar, boolean, int, uint, long,
+ * ulong, enum, flags, float, double, string, GdkColor and
+ * GtkAdjustment type values.  Support for GtkWidget type values is
+ * still to come.
+ *
+ * Returns: %TRUE on success.
+ */
 gboolean
 glade_xml_set_value_from_string (GParamSpec *pspec,
 				 const gchar *string,
@@ -1261,6 +1224,19 @@ glade_xml_set_value_from_string (GParamSpec *pspec,
     return ret;
 }
 
+/**
+ * glade_standard_build_widget
+ * @xml: the GladeXML object.
+ * @widget_type: the GType of the widget.
+ * @info: the GladeWidgetInfo structure.
+ *
+ * This is the standard widget building function.  It processes all
+ * the widget properties using the standard object properties
+ * interfaces.  This function will be sufficient for most widget
+ * types, thus reducing the ammount of work needed to wrap a library.
+ *
+ * Returns: the constructed widget.
+ */
 GtkWidget *
 glade_standard_build_widget(GladeXML *xml, GType widget_type,
 			    GladeWidgetInfo *info)
@@ -1311,54 +1287,56 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
 /**
  * glade_standard_build_children
  * @self: the GladeXML object.
- * @w: the container widget.
+ * @parent: the container widget.
  * @info: the GladeWidgetInfo structure.
- * @longname: the long name for this widget.
  *
  * This is the standard child building function.  It simply calls
- * gtk_container_add on each child to add them to the parent.  It is
- * implemented here, as it should be useful to many GTK+ based widget
- * sets.
+ * gtk_container_add on each child to add them to the parent, and
+ * process any packing properties using the generic container packing
+ * properties interfaces.
+ *
+ * This function will be sufficient for most container widgets
+ * provided that they implement the GtkContainer child packing
+ * properties interfaces.
  */
 void
-glade_standard_build_children(GladeXML *self, GtkWidget *w,
-			      GladeWidgetInfo *info, const char *longname)
+glade_standard_build_children(GladeXML *self, GtkWidget *parent,
+			      GladeWidgetInfo *info)
 {
     gint i, j;
 
-    g_object_ref(G_OBJECT(w));
+    g_object_ref(G_OBJECT(parent));
     for (i = 0; i < info->n_children; i++) {
 	GladeWidgetInfo *childinfo = info->children[i].child;
 	GtkWidget *child;
 
 	/* handle any internal children */
 	if (info->children[i].internal_child) {
-	    glade_xml_handle_internal_child(self, w, &info->children[i],
-					    longname);
+	    glade_xml_handle_internal_child(self, parent, &info->children[i]);
 	    continue;
 	}
 
-	child = glade_xml_build_widget(self, childinfo, longname);
+	child = glade_xml_build_widget(self, childinfo);
 
 	g_object_ref(G_OBJECT(child));
 	gtk_widget_freeze_child_notify(child);
-	gtk_container_add(GTK_CONTAINER(w), child);
+	gtk_container_add(GTK_CONTAINER(parent), child);
 	for (j = 0; j < info->children[i].n_properties; j++) {
 	    const gchar *name = info->children[i].properties[j].name;
 	    GValue value = { 0 };
 	    GParamSpec *pspec;
 
 	    pspec = gtk_container_class_find_child_property(
-		G_OBJECT_GET_CLASS(w), name);
+		G_OBJECT_GET_CLASS(parent), name);
 	    if (!pspec) {
 		g_warning("unknown child property `%s' for container `%s'",
-			  name, G_OBJECT_TYPE_NAME(w));
+			  name, G_OBJECT_TYPE_NAME(parent));
 		continue;
 	    }
 
 	    if (glade_xml_set_value_from_string(pspec,
 			info->children[i].properties[j].value, &value)) {
-		gtk_container_child_set_property(GTK_CONTAINER(w), child,
+		gtk_container_child_set_property(GTK_CONTAINER(parent), child,
 						 name, &value);
 		g_value_unset(&value);
 	    }
@@ -1366,7 +1344,7 @@ glade_standard_build_children(GladeXML *self, GtkWidget *w,
 	gtk_widget_thaw_child_notify(child);
 	g_object_ref(G_OBJECT(child));
     }
-    g_object_unref(G_OBJECT(w));
+    g_object_unref(G_OBJECT(parent));
 }
 
 #ifndef ENABLE_NLS
@@ -1390,20 +1368,38 @@ glade_standard_build_children(GladeXML *self, GtkWidget *w,
 /**
  * GladeBuildChildrenFunc
  * @xml: the GladeXML object.
- * @w: this widget.
- * @info: the GladeWidgetInfo structure for this widget.
- * @longname: the long name for this widget.
+ * @parent: the parent.
+ * @info: the GladeWidgetInfo structure for the parent.
  *
  * This function signature should be used by functions that are responsible
  * for adding children to a container widget.  To create each child widget,
  * glade_xml_build_widget should be called.
  */
-
+/**
+ * GladeFindInternalChildFunc
+ * @xml: the GladeXML object.
+ * @parent: the parent widget.
+ * @childname: the name of the internal child
+ *
+ * When some composite widgets are created, a number of children are
+ * added at the same time (for example, the vbox in a GtkDialog).
+ * These widgets are identified in the XML interface file by the
+ * internal-child attribute on their &lt;child&gt; element.
+ *
+ * When libglade encounters an internal child, rather than creating a
+ * new widget instance, libglade walks up the tree until it finds the
+ * first non internal-child parent.  It then calls the
+ * find_internal_child callback for that parent's class.
+ *
+ * That callback should return the internal child corresponding to the
+ * name passed in as the third argument.
+ *
+ * Returns: the named internal child.
+ */
 /**
  * glade_xml_build_widget:
  * @self: the GladeXML object.
  * @info: the GladeWidgetInfo structure for the widget.
- * @parent_long: the long name of the parent object.
  *
  * This function is not intended for people who just use libglade.  Instead
  * it is for people extending it (it is designed to be called in the child
@@ -1420,15 +1416,13 @@ glade_standard_build_children(GladeXML *self, GtkWidget *w,
  * Returns: the newly created widget.
  */
 GtkWidget *
-glade_xml_build_widget(GladeXML *self, GladeWidgetInfo *info,
-		       const char *parent_long)
+glade_xml_build_widget(GladeXML *self, GladeWidgetInfo *info)
 {
     GladeWidgetBuildData *data;
     GType widget_type;
     GtkWidget *ret;
 
-    debug(g_message("Widget class: %s (for parent %s)", info->class,
-			parent_long?parent_long:"(null)"));
+    debug(g_message("Widget class: %s", info->class));
     if (!strcmp(info->class, "Placeholder")) {
 	g_warning("placeholders exist in interface description");
 	ret = gtk_label_new("[placeholder]");
@@ -1437,23 +1431,11 @@ glade_xml_build_widget(GladeXML *self, GladeWidgetInfo *info,
     }
     data = g_hash_table_lookup(widget_table, info->class);
     if (data == NULL) {
-	if (glade_xml_build_extended_widget) {
-	    char *err = NULL;
-
-	    ret = glade_xml_build_extended_widget(self, info, &err);
-	    if (!ret) {
-		g_warning("%s", err);
-		ret = gtk_label_new(err);
-		g_free(err);
-		gtk_widget_show(ret);
-	    }
-	} else {
-	    char buf[50];
-	    g_warning("unknown widget class '%s'", info->class);
-	    g_snprintf(buf, 49, "[a %s]", info->class);
-	    ret = gtk_label_new(buf);
-	    gtk_widget_show(ret);
-	}
+	char buf[50];
+	g_warning("unknown widget class '%s'", info->class);
+	g_snprintf(buf, 49, "[a %s]", info->class);
+	ret = gtk_label_new(buf);
+	gtk_widget_show(ret);
     } else {
 	if (!data->typecode && data->get_type_func)
 	    data->typecode = data->get_type_func();
@@ -1461,14 +1443,25 @@ glade_xml_build_widget(GladeXML *self, GladeWidgetInfo *info,
 	g_assert(data->typecode);
 	ret = data->new(self, data->typecode, info);
     }
-    glade_xml_set_common_params(self, ret, info, parent_long);
+    glade_xml_set_common_params(self, ret, info);
     return ret;
 }
 
+/**
+ * glade_xml_handle_internal_child
+ * @self: the GladeXML object.
+ * @parent: the parent widget.
+ * @child_info: the GladeChildInfo structure for the child.
+ *
+ * This function is intended to be called by the build_children
+ * callback for container widgets.  If the build_children callback
+ * encounters a child with the internal-child attribute set, then it
+ * should call this function to handle it and then continue on to the
+ * next child.
+ */
 void
 glade_xml_handle_internal_child(GladeXML *self, GtkWidget *parent,
-				GladeChildInfo *child_info,
-				const gchar *parent_long)
+				GladeChildInfo *child_info)
 {
     GladeWidgetBuildData *parent_build_data = NULL;
     GtkWidget *child;
@@ -1502,16 +1495,31 @@ glade_xml_handle_internal_child(GladeXML *self, GtkWidget *parent,
 	return;
     }
 
-    glade_xml_set_common_params(self, child, child_info->child, parent_long);
+    glade_xml_set_common_params(self, child, child_info->child);
 }
 
 
+static void
+glade_xml_widget_destroy(GtkObject *object, GladeXML *xml)
+{
+    gchar *name;
+
+    g_return_if_fail(GTK_IS_OBJECT(object));
+    g_return_if_fail(GLADE_IS_XML(xml));
+
+    name = g_object_get_data(G_OBJECT(object), glade_xml_name_tag);
+
+    if (!name) return;
+    g_hash_table_remove(xml->priv->name_hash, name);
+    g_object_set_data(G_OBJECT(object), glade_xml_tag, NULL);
+    g_object_set_data(G_OBJECT(object), glade_xml_name_tag, NULL);
+}
+
 /**
  * glade_xml_set_common_params
  * @self: the GladeXML widget.
  * @widget: the widget to set parameters on.
  * @info: the GladeWidgetInfo structure for the widget.
- * @parent_long: the long name of the parent widget.
  *
  * This function sets the common parameters on a widget, and is responsible
  * for inserting it into the GladeXML object's internal structures.  It will
@@ -1521,11 +1529,10 @@ glade_xml_handle_internal_child(GladeXML *self, GtkWidget *parent,
  */
 void
 glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
-			    GladeWidgetInfo *info, const char *parent_long)
+			    GladeWidgetInfo *info)
 {
     GList *tmp;
     GladeWidgetBuildData *data;
-    char *w_longname;
 
     /* get the build data */
     if (!widget_table)
@@ -1534,25 +1541,6 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
     glade_xml_add_signals(self, widget, info);
     glade_xml_add_accels(self, widget, info);
 
-#if 0
-    for (tmp = self->priv->focus_ulines; tmp; tmp = tmp->next) {
-	GladeFocusULine *uline = tmp->data;
-
-	if (!strcmp(uline->widget_name, info->name)) {
-	    /* it is for us ... */
-	    gtk_widget_add_accelerator(widget, "grab_focus",
-				       glade_xml_ensure_accel(self),
-				       uline->key, GDK_MOD1_MASK, 0);
-	    tmp = tmp->next;
-	    self->priv->focus_ulines =
-		g_list_remove(self->priv->focus_ulines, uline);
-	    g_free(uline);
-	}
-	if (!tmp)
-	    break;
-    }
-#endif
-
     gtk_widget_set_name(widget, info->name);
 #if 0
     if (info->tooltip) {
@@ -1561,197 +1549,22 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 			     glade_xml_gettext(self, info->tooltip),
 			     NULL);
     }
-
-    gtk_widget_set_usize(widget, info->width, info->height);
-    if (info->border_width > 0)
-	gtk_container_set_border_width(GTK_CONTAINER(widget),
-				       info->border_width);
-    gtk_widget_set_sensitive(widget, info->sensitive);
-    if (info->can_default)
-	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
-    if (info->can_focus)
-	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS);
-    else
-	GTK_WIDGET_UNSET_FLAGS(widget, GTK_CAN_FOCUS);
-    if (info->has_default)
-	self->priv->default_widget = widget;
-    if (info->has_focus)
-	self->priv->focus_widget = widget;
-
-    for (tmp = info->attributes; tmp != NULL; tmp = tmp->next) {
-	GladeAttribute *attr = tmp->data;
-
-	if (!strcmp(attr->name, "events")) {
-	    char *tmpptr, *endptr;
-	    long events = strtol(attr->value, &endptr, 0);
-
-	    /* format conversion error */
-	    if (attr->value == endptr) {
-		events = 0;
-		tmpptr = attr->value;
-		while ((endptr = strchr(tmpptr, ' '))) {
-		    char *str = g_strndup(tmpptr, endptr-tmpptr);
-		    events |= glade_enum_from_string(GDK_TYPE_EVENT_MASK, str);
-		    g_free(str);
-		    tmpptr = endptr;
-		    while (tmpptr[0] == ' ' || tmpptr[0] == '|')
-			tmpptr++;
-		}
-		events |= glade_enum_from_string(GDK_TYPE_EVENT_MASK, tmpptr);
-	    }
-	    gtk_widget_set_events(widget, events);
-	} else if (!strcmp(attr->name, "extension_events")) {
-	    GdkExtensionMode ex =
-		glade_enum_from_string(GDK_TYPE_EXTENSION_MODE, attr->value);
-	    gtk_widget_set_extension_events(widget, ex);
-	}
-    }
 #endif
 
-    if (parent_long)
-	w_longname = g_strconcat(parent_long, ".", info->name, NULL);
-    else
-	w_longname = g_strdup(info->name);
-    /* store this information as data of the widget.  w_longname is
-     * owned by the widget now */
+    /* store this information as data of the widget. */
     gtk_object_set_data(GTK_OBJECT(widget), glade_xml_tag, self);
-    gtk_object_set_data(GTK_OBJECT(widget), glade_xml_name_tag,info->name);
-    gtk_object_set_data_full(GTK_OBJECT(widget), glade_xml_longname_tag,
-			     w_longname, (GtkDestroyNotify)g_free);
+    gtk_object_set_data(GTK_OBJECT(widget), glade_xml_name_tag, info->name);
     /* store widgets in hash table, for easy lookup */
     g_hash_table_insert(self->priv->name_hash, info->name, widget);
-    g_hash_table_insert(self->priv->longname_hash, w_longname, widget);
+
+    /* set up function to remove widget from GladeXML object's
+     * name_hash on destruction. Use connect_object so the handler is
+     * automatically removed on finalization of the GladeXML
+     * object. */
+    g_signal_connect_object(G_OBJECT(widget), "destroy",
+			    G_CALLBACK(glade_xml_widget_destroy),
+			    G_OBJECT(self), 0);
 
     if (data && data->build_children && info->children)
-	data->build_children(self, widget, info, w_longname);
-#if 0
-    if (info->visible)
-	gtk_widget_show(widget);
-#endif
+	data->build_children(self, widget, info);
 }
-
-/**
- * glade_get_adjustment
- * @info: the GladeWidgetInfo structure for this widget.
- *
- * This utility routine is used to create an adjustment object for a widget.
- *
- * Returns: the newly created GtkAdjustment.
- */
-GtkAdjustment *
-glade_get_adjustment(GladeWidgetInfo *info)
-{
-    gint i;
-    gdouble hvalue=1, hlower=0, hupper=100, hstep=1, hpage=100, hpage_size=10;
-
-    for (i = 0; i < info->n_properties; i++) {
-	GladeProperty *prop = &info->properties[i];
-	gchar *name = prop->name;
-
-	if (name[0] == 'h')
-	    name++;
-
-	switch (name[0]) {
-	case 'l':
-	    if (!strcmp(name, "lower"))
-		hlower = g_strtod(prop->value, NULL);
-	    break;
-	case 'p':
-	    if (!strcmp(name, "page"))
-		hpage = g_strtod(prop->value, NULL);
-	    else if (!strcmp(name, "page_size"))
-		hpage_size=g_strtod(prop->value, NULL);
-	    break;
-	case 's':
-	    if (!strcmp(name, "step"))
-		hstep = g_strtod(prop->value, NULL);
-	    break;
-	case 'u':
-	    if (!strcmp(name, "upper"))
-		hupper = g_strtod(prop->value, NULL);
-	    break;
-	case 'v':
-	    if (!strcmp(name, "value"))
-		hvalue = g_strtod(prop->value, NULL);
-	    break;
-	}
-    }
-    return GTK_ADJUSTMENT (gtk_adjustment_new(hvalue, hlower, hupper,
-					      hstep, hpage, hpage_size));
-}
-
-#if 0
-/**
- * glade_xml_set_window_props
- * @window: the GtkWindow to set the properties of.
- * @info: the GladeWidgetInfo structure holding the properties
- *
- * This is a convenience function to set some common attributes on
- * GtkWindow widgets and widgets derived from GtkWindow.  It does not
- * set the title or the window type, as these may need special handling
- * in some widgets.
- */
-void
-glade_xml_set_window_props(GtkWindow *window, GladeWidgetInfo *info)
-{
-    gint i;
-    gboolean allow_grow = window->allow_grow;
-    gboolean allow_shrink = window->allow_shrink;
-    gboolean auto_shrink = window->auto_shrink;
-    gchar *wmname = NULL, *wmclass = NULL;
-
-    for (i = 0; i < info->n_properties; i++) {
-	GladeProperty *prop = &info->properties[i];
-
-	switch (prop->name[0]) {
-	case 'a':
-	    if (!strcmp(prop->name, "allow_grow"))
-		allow_grow = prop->value[0] == 'T';
-	    else if (!strcmp(prop->name, "allow_shrink"))
-		allow_shrink = prop->value[0] == 'T';
-	    else if (!strcmp(prop->name, "auto_shrink"))
-		auto_shrink = prop->value[0] == 'T';
-	    break;
-	case 'd':
-	    if (!strcmp(prop->name, "default_height"))
-		gtk_window_set_default_size(window,
-					    -2, strtol(prop->value, NULL, 0));
-	    else if (!strcmp(prop->name, "default_width"))
-		gtk_window_set_default_size(window,
-					    strtol(prop->value, NULL, 0), -2);
-	    break;
-	case 'm':
-	    if (!strcmp(prop->name, "modal"))
-		gtk_window_set_modal(window, prop->value[0] == 'T');
-	    break;
-	case 'p':
-	    if (!strcmp(prop->name, "position"))
-		gtk_window_set_position(window,	glade_enum_from_string(
-						GTK_TYPE_WINDOW_POSITION,
-						prop->value));
-	    break;
-	case 'w':
-	    if (!strcmp(prop->name, "wmclass_name"))
-		wmname = prop->value;
-	    else if (!strcmp(prop->name, "wmclass_class"))
-		wmclass = prop->value;
-	    break;
-	case 'x':
-	    if (prop->name[1] == '\0')
-		gtk_widget_set_uposition(GTK_WIDGET(window),
-					 strtol(prop->value, NULL, 0), -2);
-	    break;
-	case 'y':
-	    if (prop->name[1] == '\0')
-		gtk_widget_set_uposition(GTK_WIDGET(window),
-					 -2, strtol(prop->value, NULL, 0));
-	    break;
-	}
-    }
-    gtk_window_set_policy(window, allow_shrink, allow_grow, auto_shrink);
-    if (wmname != NULL || wmclass != NULL)
-	gtk_window_set_wmclass(window,
-			       wmname  ? wmname  : "",
-			       wmclass ? wmclass : "");
-}
-#endif
